/* binomial.cpp  by Wolf Jung (C) 2009, 10, 12.  Defines classes:
   QmnBinomi, QmnColumns, QmnBinDemo, QmnBinDialog, binomial

   These classes are part of Binomial 2.2, which is free software; you can
   redistribute and / or modify them under the terms of the GNU General
   Public License as published by the Free Software Foundation; either
   version 3, or (at your option) any later version. In short: there is
   no warranty of any kind; you must redistribute the source code as well.

&alpha;  \xce\xb1     \u03b1
&beta;   \xce\xb2     \u03b2
&gamma;  \xce\xb3     \u03b3
&theta;  \xce\xb8     \u03b8
&lambda; \xce\xbb     \u03bb
&mu;     \xce\xbc     \u03bc
&pi;     \xcf\x80     \u03c0
&rho;    \xcf\x81     \u03c1
&sigma;  \xcf\x83     \u03c3
&phi;    \xcf\x86     \u03c6
&Phi;    \xce\xa6     \u03a6
&le;     \xe2\x89\xa4 \u2264
&ge;     \xe2\x89\xa5 \u2265
&radic;  \xe2\x88\x9a \u221a
&ldquo;  \xe2\x80\x9c \u201c
&rdquo;  \xe2\x80\x9d \u201d
*/

#include <QtGui>
#if (QT_VERSION >= 0x050000)
#include <QtWidgets>
#endif
#define USING_BinIcon_xpm16
#define USING_BinIcon_xpm64
#include "binomial.h"

void QmnBinomi::about()
{  QMessageBox mb(trUtf8("About Binomial"), trUtf8(
   "<style type=\"text/css\">a:link { text-decoration: none; }</style>"
   "<b>Binomial 2.2</b> of December 21, 2012.<br>"
   "Copyright &copy; Wolf Jung"
   "<p>This program comes with no warranty.<br>"
   "It may be redistributed and modified under<br>"
   "the terms of the "
   "<a href=\"http://www.gnu.org/copyleft/gpl.html\">"
   "GNU General Public Licence</a>."
   "<p>The latest version and the source code are<br>"
   "available from "
   "<a href=\"http://www.mndynamics.com\">www.mndynamics.com</a>"),
   QMessageBox::Information,
   QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton, this);
   mb.setIconPixmap(QPixmap((const char **)(BinIcon_xpm64)));
   mb.exec();
}

int main(int argc, char *argv[])
{  QApplication app(argc, argv);
   QString locale = QLocale::system().name();
   QLocale::setDefault(QLocale(locale));
   if (argc > 1) locale = argv[1];
   QTranslator myTranslator, qtTranslator;
   if (!locale.contains("en")
       && myTranslator.load("binom_" + locale, qApp->applicationDirPath()))
   {  app.installTranslator(&myTranslator);
      qtTranslator.load("qt_" + locale, qApp->applicationDirPath());
      app.installTranslator(&qtTranslator);
   }
   QmnBinomi *shell = new QmnBinomi();
   shell->show();
   return app.exec();
}

QmnBinomi::QmnBinomi(QWidget *parent) : QMainWindow(parent)
{  setWindowIcon(QPixmap((const char **)(BinIcon_xpm16)));
   mode = 0; binomi = new binomial();
   coords = new QLineEdit(QString(" "), 0);
   coords->setReadOnly(true); coords->setFocusPolicy(Qt::ClickFocus);
#if (QT_VERSION >= 0x040600) && (QT_VERSION < 0x040700)
//see  https://bugreports.qt-project.org/browse/QTBUG-7395
   coords->setFocusPolicy(Qt::NoFocus);
#endif
   columns = new QmnColumns(640, 480); //size is ignored
   connect(columns, SIGNAL(resized(int, int)), this, SLOT(redraw()));
   connect(columns, SIGNAL(mouseMoved(double)), this, SLOT(updateX(double)));
   demo = new QmnBinDemo(0); demo->hide();
   QWidget *dummy = new QWidget(this); setCentralWidget(dummy);
   QVBoxLayout *vLayout = new QVBoxLayout;
   vLayout->addWidget(coords);
   vLayout->addWidget(columns);
   dummy->setLayout(vLayout);

   QActionGroup *fileAG = new QActionGroup(this);
   pngSaveAct = new QAction(trUtf8("&Save .png ..."), fileAG);
   pngSaveAct->setShortcut(Qt::Key_F7);
   QAction *fSeparator = new QAction(this);
   fSeparator->setSeparator(true); fileAG->addAction(fSeparator);
   closeAct = new QAction(trUtf8("&Close"), fileAG);
   closeAct->setShortcut(Qt::ALT+Qt::Key_F4);
   exitAct = new QAction(trUtf8("&Exit"), fileAG);
   exitAct->setShortcut(QString("Ctrl+q"));
   connect(fileAG, SIGNAL(triggered(QAction*)), this, SLOT(file(QAction*)));
   this->addActions(fileAG->actions());

   QActionGroup *drawAG = new QActionGroup(this);
   pAct = new QAction(trUtf8("&Probability ..."), drawAG);
   pAct->setShortcut(QString("p"));
   nAct = new QAction(trUtf8("&Sample size ..."), drawAG);
   nAct->setShortcut(QString("n"));
   QAction *dSeparator = new QAction(this);
   dSeparator->setSeparator(true); drawAG->addAction(dSeparator);
   cumulAct = new QAction(trUtf8("&Cumulative?"), drawAG);
   cumulAct->setShortcut(Qt::Key_F2);
   scaleAct = new QAction(trUtf8("&Rescaled?"), drawAG);
   scaleAct->setShortcut(Qt::Key_F3);
   curveAct = new QAction(trUtf8("&Gaussian curve?"), drawAG);
   curveAct->setShortcut(Qt::Key_F4);
   connect(drawAG, SIGNAL(triggered(QAction*)), this, SLOT(draw(QAction*)));
   this->addActions(drawAG->actions());

   QActionGroup *compAG = new QActionGroup(this);
   xAct = new QAction(trUtf8("Compute P(&x) ..."), compAG);
   xAct->setShortcut(QString("x"));
   zAct = new QAction(trUtf8("Compute \xce\xa6(&z) ..."), compAG);
   zAct->setShortcut(QString("z"));
   QAction *cSeparator = new QAction(this);
   cSeparator->setSeparator(true); compAG->addAction(cSeparator);
   testAct = new QAction(trUtf8("&Significance ..."), compAG);
   testAct->setShortcut(QString("s"));
   confiAct = new QAction(trUtf8("&Confidence ..."), compAG);
   confiAct->setShortcut(QString("c"));
   connect(compAG, SIGNAL(triggered(QAction*)), this, SLOT(comp(QAction*)));
   this->addActions(compAG->actions());

   QActionGroup *helpAG = new QActionGroup(this);
   demoActs[0] = new QAction(trUtf8("&Help ..."), helpAG);
   demoActs[0]->setShortcut(Qt::Key_F1);
   QAction *hSeparator = new QAction(this);
   hSeparator->setSeparator(true); helpAG->addAction(hSeparator);
   demoActs[1] = new QAction(trUtf8("Page &1: binomial distribution ..."),helpAG);
   demoActs[1]->setShortcut(QString("1"));
   demoActs[2] = new QAction(trUtf8("Page &2: applications ..."), helpAG);
   demoActs[2]->setShortcut(QString("2"));
   demoActs[3] = new QAction(trUtf8("Page &3: \xce\xbc and \xcf\x83 ..."),
      helpAG);
   demoActs[3]->setShortcut(QString("3"));
   demoActs[4] = new QAction(trUtf8("Page &4: \xcf\x83-neighborhoods ..."),
      helpAG);
   demoActs[4]->setShortcut(QString("4"));
   demoActs[5] = new QAction(trUtf8("Page &5: approximation ..."), helpAG);
   demoActs[5]->setShortcut(QString("5"));
   demoActs[6] = new QAction(trUtf8("Page &6: binomial test ..."), helpAG);
   demoActs[6]->setShortcut(QString("6"));
   demoActs[7] = new QAction(trUtf8("Page &7: confidence interval ..."), helpAG);
   demoActs[7]->setShortcut(QString("7"));
   demoActs[8] = new QAction(trUtf8("Page &8: method of computation ..."),helpAG);
   demoActs[8]->setShortcut(QString("8"));
   QAction *HSeparator = new QAction(this);
   HSeparator->setSeparator(true); helpAG->addAction(HSeparator);
   aboutAct = new QAction(trUtf8("&About ..."), helpAG);
   qtAct = new QAction(trUtf8("About &Qt ..."), helpAG);
   connect(helpAG, SIGNAL(triggered(QAction*)), this, SLOT(help(QAction*)));
   this->addActions(helpAG->actions());

   QMenu *fileMenu, *drawMenu, *compMenu, *helpMenu;
   fileMenu = menuBar()->addMenu(trUtf8("&File"));
   fileMenu->addActions(fileAG->actions());
   drawMenu = menuBar()->addMenu(trUtf8("&Draw"));
   drawMenu->addActions(drawAG->actions());
   compMenu = menuBar()->addMenu(trUtf8("&Compute"));
   compMenu->addActions(compAG->actions());
   helpMenu = menuBar()->addMenu(trUtf8("&Help"));
   helpMenu->addActions(helpAG->actions());

   updateValues(); updateX(0);

   confiAct->setEnabled(false);
   for (int i = 7; i <= 8; i++) demoActs[i]->setEnabled(false);
}//QmnBinomi

void QmnBinomi::updateValues()
{  QLocale locale = QLocale::system();
#if QT_VERSION >= 0x040200
   locale.setNumberOptions(QLocale::OmitGroupSeparator);
#endif //QLocale::toString() is locale-aware, QString::number() is not.
   coords->setText(trUtf8(
      "p = %1   n = %2          \xce\xbc = %3   \xcf\x83 = %4").arg(
      locale.toString(binomi->get_p(), 'g', 6)).arg(
      QString::number(binomi->get_n())).arg(
      locale.toString(binomi->get_mu(), 'g', 6)).arg(
      locale.toString(binomi->get_sigma(), 'g', 6)));
   QString txt = QString();
   if (mode & 2) txt = trUtf8(", rescaled -3.0 < z < 3.0");
   if (mode & 1)
      setWindowTitle(trUtf8("Binomial: cumulative distribution") + txt); 
   else setWindowTitle(trUtf8("Binomial: probability distribution") + txt);
}

void QmnBinomi::updateX(double x)
{  QLocale locale = QLocale::system(); int k;
#if QT_VERSION >= 0x040200
   locale.setNumberOptions(QLocale::OmitGroupSeparator);
#endif //QLocale::toString() is locale-aware, QString::number() is not.
   if (mode & 2)
   {  x = 6*x - 3.0;
      k = int(binomi->get_mu() + 0.5 + x*binomi->get_sigma());
   }
   else
   {  x *= double(binomi->get_n() + 1); k = int(x);
      x = (x - 0.5 - binomi->get_mu())/binomi->get_sigma();
   }
   statusBar()->showMessage(QString("k = %1   z = %2").arg(
      QString::number(k)).arg(locale.toString(x, 'g', 6)));
}

void QmnBinomi::redraw()
{ columns->draw(binomi, mode); }

void QmnBinomi::file(QAction *act)
{  if (act == pngSaveAct)
   {  QString name = QFileDialog::getSaveFileName(this,
         trUtf8("Binomial: save PNG"), "png", "Images (*.png)");
      if (name.isEmpty()) return;
      if (!name.contains(".png", Qt::CaseInsensitive)) name += ".png";
      columns->savePNG(name);
   }
   if (act == closeAct) close();
   if (act == exitAct) qApp->closeAllWindows();
}

void QmnBinomi::draw(QAction *act)
{  if (act == pAct)
   {  double x = binomi->get_p();
      QmnBinDialog *dialog = new QmnBinDialog(trUtf8(
         "Enter the probability p of success:"), &x, this);
      if (!dialog->exec() || x <= 0 || x >= 1.0) return;
      mode = 0; binomi->set_p(x); updateValues();
      columns->draw(binomi, mode, 0);
   }
   if (act == nAct)
   {  bool ok;
#if (QT_VERSION >= 0x040500)      
      int n = QInputDialog::getInt(
#else
      int n = QInputDialog::getInteger(
#endif
         0, "Binomial", trUtf8(
         "Enter the sample size n (10 ... 1000) :"),
         binomi->get_n(), 10, 1000, 1, &ok);
      if (!ok) return;
      binomi->set_n(n); updateValues(); columns->draw(binomi, mode, 0);
   }
   if (act == cumulAct)
   { mode ^= 1; updateValues(); columns->draw(binomi, mode, 0); }
   if (act == scaleAct)
   { mode ^= 2; updateValues(); columns->draw(binomi, mode, 0); }
   if (act == curveAct)
   { mode ^= 4; columns->draw(binomi, mode); }
}//draw

void QmnBinomi::comp(QAction *act)
{  if(act == xAct)
   {  bool ok;
#if (QT_VERSION >= 0x040500)      
      int k = QInputDialog::getInt(
#else
      int k = QInputDialog::getInteger(
#endif
         0, "Binomial", trUtf8(
         "Enter the number X of successes, 0 \xe2\x89\xa4 X \xe2\x89\xa4 %1 :").arg(
            QString::number(binomi->get_n())),
         0, 0, binomi->get_n(), 1, &ok);
      if (!ok) return;
      QLocale locale = QLocale::system();
#if QT_VERSION >= 0x040200
      locale.setNumberOptions(QLocale::OmitGroupSeparator);
      double z = (k + 0.5 - binomi->get_mu())/binomi->get_sigma(),
         Phi = binomial::Phi(z), Q = binomi->get_Q(k);
      QMessageBox::information(this, "Binomial", trUtf8(
         "Probability of %1 successes:\n"
         "    P(X = %1) = %2\n"
         "Cumulated probability:\n"
         "   P(X \xe2\x89\xa4 %1) = %3\n"
         "    P(X > %1) = %4\n"
         "Approximation X = %1.5 ~  z = %5 :\n"
         "    \xce\xa6(%5) = %6\n"
         "1 - \xce\xa6(%5) = %7").arg(
            QString::number(k)).arg(
            locale.toString(binomi->get_P(k), 'g', 6)).arg(
            locale.toString(Q, 'g', 6)).arg(
            locale.toString(1.0 - Q, 'g', 6)).arg(
            locale.toString(z, 'g', 6)).arg(
            locale.toString(Phi, 'g', 6)).arg(
            locale.toString(1.0 - Phi, 'g', 6)),
      QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
      QMessageBox::NoButton);
#endif
   }
   if(act == zAct)
   {  double x = 0; QmnBinDialog *dialog = new QmnBinDialog(trUtf8(
         "To compute the Gaussian distribution,\n"
         "enter z with -3.0 \xe2\x89\xa4 z \xe2\x89\xa4 3.0 :"), &x, this);
       if (!dialog->exec()) return;
      QLocale locale = QLocale::system();
#if QT_VERSION >= 0x040200
      double Phi = binomial::Phi(x);
      QMessageBox::information(this, "Binomial", trUtf8(
         "Gaussian probability distribution:\n"
         "    \xcf\x86(%1) = %2\n"
         "Cumulated Gaussian distribution :\n"
         "    \xce\xa6(%1) = %3\n"
         "1 - \xce\xa6(%1) = %4").arg(
            locale.toString(x, 'g', 6)).arg(
            locale.toString(binomial::phi(x), 'g', 6)).arg(
            locale.toString(Phi, 'g', 6)).arg(
            locale.toString(1.0 - Phi, 'g', 6)),
      QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
      QMessageBox::NoButton);
#endif
   }
   if(act == testAct)
   {  double x = 0.05; QmnBinDialog *dialog = new QmnBinDialog(trUtf8(
         "Enter the significance level \xce\xb1 for\n"
         "the one-tailed test, 0 < \xce\xb1 \xe2\x89\xa4 0.2 :"), &x, this);
      if (!dialog->exec() || x < 0 || x > 0.2) return;
      if (x <= 0) { columns->draw(binomi, mode, 0); return; }
      QLocale locale = QLocale::system();
#if QT_VERSION >= 0x040200
      int k, n = binomi->get_n(); QString txtl, txtr;
      if (binomi->get_P(0) > x)
         txtl = trUtf8("No solution for the left hand tail.\n");
      else for (k = 1; k <= n; k++) if (binomi->get_Q(k) > x)
      {  txtl = trUtf8("P(X \xe2\x89\xa4 %1) = %2\n").arg(
                QString::number(k - 1)).arg(
                locale.toString(binomi->get_Q(k - 1), 'g', 6));
         break;
      }
      if (binomi->get_P(n) > x)
         txtr = trUtf8("No solution for the right hand tail.\n\n");
      else for (k = n - 1; k >= 0; k--) if (binomi->get_Q(k) < 1.0 - x)
      {  txtr = trUtf8("P(X > %1) = %2\n\n").arg(
                QString::number(k + 1)).arg(
                locale.toString(1.0 - binomi->get_Q(k + 1), 'g', 6));
         break;
      }
      double z = binomial::invPhi(1.0  - x),
         m = binomi->get_mu(), zs = z*binomi->get_sigma();
      if (QMessageBox::question(this, "Binomial", trUtf8(
      "One-tailed binomial test with the\n"
      "significance level \xce\xb1 = %1 :\n").arg(locale.toString(x))
      + txtl + txtr + trUtf8(
      "Gaussian approximation z = %1\n"
      "gives k < %2 or k > %3 .\n\n"
      "Do you want to mark the tails?").arg(
          locale.toString(z, 'g', 6)).arg(
          locale.toString(m - zs, 'g', 6)).arg(
          locale.toString(m + zs, 'g', 6)),
      QMessageBox::Yes | QMessageBox::Default,
      QMessageBox::No | QMessageBox::Escape) == QMessageBox::Yes)
      { mode &= 60; columns->draw(binomi, mode, x); }
#endif
   }
}//comp

void QmnBinomi::help(QAction *act)
{  for (int i = 0; i <= 8; i++) if (act == demoActs[i]) demo->showPage(i);
   if (act == aboutAct) about();
   if (act == qtAct) qApp->aboutQt();
}//help


QmnColumns::QmnColumns(int w, int h, QWidget *parent)
   : QWidget(parent)
{  setAttribute(Qt::WA_OpaquePaintEvent);
   setFocusPolicy(Qt::NoFocus);
   s = 0; buffer = 0; imax = 0; kmax = 0;
   setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
   resize(w, h); setMouseTracking(true); setCursor(Qt::CrossCursor);
}

QSize QmnColumns::sizeHint()
{ return QSize(imax, kmax); }

void QmnColumns::resize(int w, int h)
{  if (imax > 0 && imax == w && kmax == h) return;
   imax = w; kmax = h;
   if (imax < 320) imax = 320; if (kmax < 240) kmax = 240;
   imax -= (imax & 7); //imax is divisible by 8
   kmax -= (kmax & 3); //kmax is divisible by 4
   setMinimumSize(320, 240);
   delete buffer; buffer = new QImage(imax, kmax, QImage::Format_RGB32);
   QWidget::resize(imax, kmax);
   //updateGeometry();
   emit resized(imax, kmax); 
}

void QmnColumns::paintEvent(QPaintEvent *event)
{  QPainter p(this);
   p.drawImage(event->rect().topLeft(), *buffer, event->rect());
}

void QmnColumns::resizeEvent(QResizeEvent *event)
{  int w = width(), h = height(); resize(w, h);
}

void QmnColumns::mouseMoveEvent(QMouseEvent *event)
{ emit mouseMoved(double(event->x())/double(imax - 1)); }

bool QmnColumns::savePNG(const QString &fileName)
{ return buffer->save(fileName, "PNG"); }

void QmnColumns::draw(binomial *b, int mode, double alpha) //= -1.0
{  if (alpha >= 0 && alpha < 0.5) s = alpha;
   QColor cl = (mode & 2 ? Qt::yellow : Qt::white); buffer->fill(cl.rgba());
   QPainter *p = new QPainter(buffer); p->setPen(Qt::lightGray);
   int i, k, n = b->get_n();
   double x1, fx, fy, P = b->get_mu(), sigma = b->get_sigma();
   if (mode & 2) fy = 0.45; else fy = 0.1;
   x1 = P + b->get_p(); fx = b->get_P(int(x1));
   if (mode & 2) fx /= sigma; if (fx > fy) fy = fx;
   fx = b->get_P(int(x1 - 1));
   if (mode & 2) fx /= sigma; if (fx > fy) fy = fx;
   if (!(mode & 2)) fx = 1.0/(sqrt(2*M_PI)*sigma); if (fx > fy) fy = fx;
   if (mode & 1) fy = 1.0;
   fy = double(kmax - 2)/fy; //pixel at ((x - x1)/fx, kmax - 1 - fy*y)
   if (mode & 2)
   {  x1 = P - 3*sigma; fx = (6*sigma)/double(imax - 1);
      if (!(mode & 1)) for (k = -2; k <= 2; k++)
      { i = ((3 + k)*imax)/6; p->drawLine(i, 0, i, kmax - 1); }
   }
   else
   {  x1 = -0.5; fx = double(n + 1)/double(imax - 1);
      if (!(mode & 1)) for (k = -3; k <= 3; k++)
      { i = int((P + k*sigma - x1)/fx); p->drawLine(i, 0, i, kmax - 1); }
   }
   for (i = 0; i < imax; i++)
   {  P = b->diagram(mode & 1, x1 + fx*i, k); if (k < 0) continue;
      cl = (k & 1? Qt::red : Qt::darkRed);
      if (!(mode & 1) && (mode & 2))  P *= sigma;
      if (!(mode & 3))
      {  if (s > 0 && (b->get_Q(k) < s || b->get_Q(k) - P > 1.0 - s) )
            cl = (k & 1? Qt::green : Qt::darkGreen);
      }
      p->setPen(cl); p->drawLine(i, int(kmax - 1 - fy*P), i, kmax - 1);
   }
   if (mode & 4)
   {  fx /= sigma; p->setPen(Qt::blue);
      if (mode & 2) x1 = -3.0;
      else { x1 = -(0.5 + b->get_mu())/sigma; if (!(mode & 1)) fy /= sigma; }
      if (mode & 1) n = int(fy*b->Phi(x1)); else n = int(fy*b->phi(x1));
      for (i = 1; i < imax; i += 2)
      {  P = x1 + fx*i;
         if (mode & 1) k = int(fy*b->Phi(P)); else k = int(fy*b->phi(P));
         p->drawLine((i > 2 ? i - 2 : 0), kmax-1 - n, i, kmax-1 - k); n = k;
      }
   }
   delete p; update();
}


QmnBinDemo::QmnBinDemo(QWidget *parent) : QDialog(parent)
{  setMinimumWidth(640); /*setMaximumWidth(800);*/ setMinimumHeight(480);
   QFont Font = font();
   if (Font.pointSize() > 0 && Font.pointSize() < 12) Font.setPointSize(12);
   setFont(Font);
   text = new QTextBrowser();
#if QT_VERSION >= 0x040200
   text->setTextInteractionFlags(Qt::LinksAccessibleByMouse);
   text->setOpenExternalLinks(false);
#endif
#if QT_VERSION >= 0x040300
   text->setOpenLinks(false);
#endif
   connect(text, SIGNAL(anchorClicked(QUrl)), this, SLOT(openLink(QUrl)));
   QPushButton *closeButton = new QPushButton(trUtf8("Ok"));
   closeButton->setAutoDefault(true);
   connect(closeButton, SIGNAL(clicked()), this, SLOT(hide()));
   QPushButton *fontButton = new QPushButton(trUtf8("Font ..."));
   fontButton->setAutoDefault(false);
   connect(fontButton, SIGNAL(clicked()), this, SLOT(changeFont()));
   QHBoxLayout *hLayout = new QHBoxLayout;
   hLayout->addWidget(closeButton);
   hLayout->addStretch(1);
   hLayout->addWidget(fontButton);
   QVBoxLayout *vLayout = new QVBoxLayout;
   vLayout->addWidget(text);
   vLayout->addLayout(hLayout);
   setLayout(vLayout);
}

void QmnBinDemo::showPage(int newpage)
{  page = newpage;
   if (page < 1 || page > 7)
   {  setWindowTitle(trUtf8("Binomial: help"));
      text->setHtml(trUtf8(
 "The program <b>Binomial</b> serves to draw the binomial probability "
 "distribution and to do related calculations.<br>"
 "An introduction to the theory is available with the keys 1 to 8. "
 "Other keys:<p>"
 "F1 help.<br>"
 "F2 cumulative ?"
 "<br>F3 rescaled (z instead of X) ?<br>"
 "F4 with Gaussian curve ?<br>"
 "F7 save image as *.png .<br>"
 "[p] probability p of success.<br>"
 "[n] sample size n.<br>"
 "[x] compute P(X...) .<br>"
 "[z] compute P(z...) .<br>"
 "[s] compute interval of rejection.<br>"
 "[c] compute confidence interval.<br>"
 "Alt+F4, Strg+q quit.<p>"
 "Decimal numbers may be entered with dot or comma as the "
 "decimal separator, or as a fraction.<br>"
 "Choose the language with a command-line parameter, "
 "\"binomial en\" or \"binomial de.\" Or in a shortcut under Windows: "
 "\"C:\\...\\binomial.exe\" de .<p>"
 "<i>Enjoy!</i>"));
   }
   if (page == 1)
   {  setWindowTitle(trUtf8("Binomial: page 1, binomial distribution"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 1: binomial distribution &rarr; see German version</i>"));
   }
   if (page == 2)
   {  setWindowTitle(trUtf8("Binomial: page 2, applications"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 2: applications &rarr; see German version</i>"));
   }
   if (page == 3)
   {  setWindowTitle(trUtf8("Binomial: page 3, \xce\xbc and \xcf\x83"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 3: &mu; and &sigma; &rarr; see German version</i>"));
   }
   if (page == 4)
   {  setWindowTitle(trUtf8("Binomial: page 4, \xcf\x83-neighborhoods"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 4: &sigma;-neighborhoods &rarr; see German version</i>"));
   }
   if (page == 5)
   {  setWindowTitle(trUtf8("Binomial: page 5, approximation"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 5: approximation &rarr; see German version</i>"));
   }
   if (page == 6)
   {  setWindowTitle(trUtf8("Binomial: page 6, binomial test"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 6: binomial test &rarr; see German version</i>"));
   }
   if (page == 7)
   {  setWindowTitle(trUtf8("Binomial: page 7, confidence interval"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 7: confidence interval &rarr; see German version</i>"));
   }
   if (page == 8)
   {  setWindowTitle(trUtf8("Binomial: page 8, method of computation"));
      text->setHtml(trUtf8(
 ""
 "<p>"
 "<i>Page 8: method of computation &rarr; see German version</i>"));
   }
   show(); raise();
}//showPage

void QmnBinDemo::changeFont()
{ setFont(QFontDialog::getFont(0, font())); raise(); }

void QmnBinDemo::openLink(const QUrl link)
{  QString string = link.toString();
#if QT_VERSION >= 0x040200
   if (string.contains("javascript"))
      QMessageBox::information(this, "Binomial",
         string.remove("javascript:alert('").remove("')"));
   else if (QMessageBox::question(this, "Binomial", trUtf8(
      "<style type=\"text/css\">a:link{text-decoration: none;}</style>"
      "Do you want to open an external link:<br>"
      "<a href=\"%1\">%1</a>").arg(string),
      QMessageBox::Yes | QMessageBox::Default,
      QMessageBox::No | QMessageBox::Escape)
         == QMessageBox::Yes) QDesktopServices::openUrl(link);
#endif
}


QmnBinDialog::QmnBinDialog(QString text, double *x, QWidget *parent)
   : QDialog(parent)
{  setAttribute(Qt::WA_DeleteOnClose); setWindowTitle(QString("Binomial"));
#if QT_VERSION >= 0x040100
   setWindowModality(Qt::WindowModal);
#endif
   X = x;
   QLabel *label = new QLabel(); label->setFocusPolicy(Qt::NoFocus);
   label->setText(text);
   QRegExpValidator *validator = new QRegExpValidator(QRegExp(QString(
      "^-?(?:[0-9]{1,4}"
      "|[0-9]{0,4}\\.[0-9]{1,30}"
      "|[0-9]{0,4},[0-9]{1,30}"
      "|[0-9]{1,15}/[1-9]{1,1}[0-9]{0,14})$")), this);
   edit = new QLineEdit(); edit->setValidator(validator);
   edit->setText(QString::number(*X, 'f', 8));
   if (!edit->hasAcceptableInput()) edit->setText(QString("0"));

   QPushButton *button2;
   button1 = new QPushButton(trUtf8("Ok"));
   button1->setShortcut(Qt::Key_Return);
   connect(button1, SIGNAL(clicked()), this, SLOT(inputFinished()));
   button2 = new QPushButton(trUtf8("Cancel"));
   button2->setShortcut(Qt::Key_Escape);
   connect(button2, SIGNAL(clicked()), this, SLOT(reject()));

   connect(edit, SIGNAL(textEdited(QString)), this, SLOT(inputChanged()));
   connect(edit, SIGNAL(returnPressed()), this, SLOT(inputFinished()));

   QHBoxLayout *hLayout = new QHBoxLayout;
   hLayout->addWidget(button1);
   hLayout->addWidget(button2);
   QVBoxLayout *vLayout = new QVBoxLayout;
   vLayout->addWidget(label);
   vLayout->addWidget(edit);
   vLayout->addLayout(hLayout);
   setLayout(vLayout);
   edit->selectAll();
   show();
}

void QmnBinDialog::inputChanged()
{ button1->setEnabled(edit->hasAcceptableInput()); }

void QmnBinDialog::inputFinished()
{  double x, y; QString text = edit->text();
   if (text.contains(QChar('/')))
   {  QStringList numbers = text.split(QChar('/'));
      x = numbers.first().toDouble(); y = numbers.last().toDouble();
      x /= y;
   }
   else
   { text = text.replace(QChar(','), QChar('.')); x = text.toDouble(); }
   *X = x;
   accept();
}


double binomial::get_P(int k)
{ if (k < 0 || k > n) return 0; return P[k]; }

double binomial::get_Q(int k)
{ if (k < 0) return 0; if (k >= n) return 1.0; return Q[k]; }

double binomial::diagram(int cumulative, double x, int &k)
{  k = int(x + 0.5); if (k < 0 || k > n) { k = -1; return 0; }
   if (!cumulative) return P[k];
   if (k == n) return 1.0;
   return Q[k] + (x - k - 0.5)*P[k];
}

void binomial::set_n(int n0)
{  if (n0 >= 8 && n0 <= 10000) n = n0;
   delete[] P; delete[] Q;
   P = new double[n+1]; Q = new double[n+1];
   compute();
}

void binomial::set_p(double p0)
{ if (p0 > 0 && p0 < 1.0) p = p0; compute(); }

void binomial::compute()
{  P[0] = pow(1 - p, n); Q[0] = P[0];
   double Pk; int i, j, k;
   for (k = 1; k <= n; k++)
   {  Pk = pow(p, k)*pow(1 - p, n - k);
      for (i = 1, j = n; i <= (2*k > n ? n - k : k); i++, j--)
      { Pk *= j; Pk /= i; }
      P[k] = Pk; Q[k] = Q[k-1] + Pk;
   }
}

double binomial::phi(double z)
{ return exp(-0.5*z*z)/sqrt(2*M_PI); }

double binomial::Phi(double z)
{  if (z > 5.0) return 1.0; if (z < -5.0) return 0;
   int j; double t = z/sqrt(2*M_PI), s = t; z = -0.5*z*z;
   for (j = 1; j <= 44; j++)
   {  t *= z/j; s += t/(2*j + 1); }
   return s + 0.5;
}

double binomial::invPhi(double Q)
{  if (Q <= 0) return -5.0; if (Q >= 1.0) return 5.0;
   int j; double h, z = 0;
   for (j = 1; j <= 100; j++)
   {  h = (Q - Phi(z))/phi(z); z += h;
      if (h < 0) h = -h; if (h < 1e-7) break;
   }
   return z;
}
