/* binomial.h  by Wolf Jung (C) 2009, 10, 12.
   Defines icons and declares classes:
   binomial, QmnColumns, QmnBinDemo, QmnBinDialog, QmnBinomi

   These classes are part of Binomial 2.2, which is free software; you can
   redistribute and / or modify them under the terms of the GNU General
   Public License as published by the Free Software Foundation; either
   version 3, or (at your option) any later version. In short: there is
   no warranty of any kind; you must redistribute the source code as well.
*/
#ifndef BINOMIAL_H_INCLUDED
#define BINOMIAL_H_INCLUDED

#include <QMainWindow>
#include <QDialog>
#include <QUrl>
#include <cmath>

class QAction;
class QLineEdit;
class QTextBrowser;
class QWidget;

class binomial {
private: //disabled
   binomial(const binomial &);
   binomial &operator=(const binomial &);
protected:
   int n;
   double p, *P, *Q;
   void compute();
public:
   binomial()
   { n = 100; p = 0.5; P = new double[n+1]; Q = new double[n+1]; compute(); }
   ~binomial() { delete[] Q; delete[] P; }
   int get_n() { return n; }
   double get_p() { return p; }
   double get_mu() { return n*p; }
   double get_sigma() { return sqrt(n*p*(1 - p)); }
   double get_P(int k);
   double get_Q(int k);
   double diagram(int cumulative, double x, int &k);
   void set_n(int n0);
   void set_p(double p0);
   static double phi(double z);
   static double Phi(double z);
   static double invPhi(double Q);
};

class QmnColumns : public QWidget
{
   Q_OBJECT
public:
   QmnColumns(int w, int h, QWidget *parent = 0);
   ~QmnColumns() { delete buffer; }
   QSize sizeHint();
   void resize(int w, int h);
   bool savePNG(const QString &fileName);
   void draw(binomial *b, int mode, double alpha = -1.0);
protected:
   int imax, kmax;
   double s;
   QImage *buffer;
   void paintEvent(QPaintEvent *event);
   void resizeEvent(QResizeEvent *event);
   void mouseMoveEvent(QMouseEvent *event);
signals:
   void resized(int w, int h);
   void mouseMoved(double x);
};

class QmnBinDemo : public QDialog
{
   Q_OBJECT
public:
   QmnBinDemo(QWidget *parent = 0);
public slots:
   void showPage(int newpage);
protected slots:
   virtual void changeFont();
   virtual void openLink(const QUrl link);
protected:
   int page;
   QTextBrowser *text;
};

class QmnBinDialog : public QDialog
{
   Q_OBJECT
public:
   QmnBinDialog(QString text, double *x, QWidget *parent);
protected slots:
   void inputChanged();
   void inputFinished();
protected:
   double *X;
   QLineEdit *edit;
   QPushButton *button1;
};

class QmnBinomi : public QMainWindow
{
   Q_OBJECT
public:
   QmnBinomi(QWidget *parent = 0);
protected slots:
   void about();
   void updateValues();
   void updateX(double x);
   void redraw();
   void file(QAction *act);
   void draw(QAction *act);
   void comp(QAction *act);
   void help(QAction *act);
protected:
   int mode;
   binomial *binomi;
   QLineEdit *coords;
   QmnColumns *columns;
   QmnBinDemo *demo;
   QAction *pngSaveAct,
      *closeAct,
      *exitAct,
      *pAct,
      *nAct,
      *cumulAct,
      *scaleAct,
      *curveAct,
      *xAct,
      *zAct,
      *testAct,
      *confiAct,
      *demoActs[9],
      *aboutAct,
      *qtAct;
};

#ifdef USING_BinIcon_xpm16
static const char *const BinIcon_xpm16[]={
"16 16 3 1",
"# c #800000",
"a c #ff0000",
". c None",
"................",
".......##.......",
".......##.......",
".......##.......",
".....aa##a......",
".....aa##a......",
".....aa##a......",
".....aa##a......",
".....aa##a......",
"....#aa##a##....",
"....#aa##a##....",
"....#aa##a##....",
"....#aa##a##....",
"..aa#aa##a##aa..",
"##aa#aa##a##aa#.",
"##aa#aa##a##aa#."};
#endif

#ifdef USING_BinIcon_xpm24
static const char *const BinIcon_xpm24[]={
"24 24 3 1",
"# c #800000",
"a c #ff0000",
". c None",
"........................",
"...........##...........",
"...........##...........",
"...........##...........",
"...........##...........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"........aaa##aaa........",
"......##aaa##aaa##......",
"......##aaa##aaa##......",
"......##aaa##aaa##......",
"......##aaa##aaa##......",
"......##aaa##aaa##......",
"......##aaa##aaa##......",
"...aaa##aaa##aaa##aaa...",
"...aaa##aaa##aaa##aaa...",
"###aaa##aaa##aaa##aaa##.",
"###aaa##aaa##aaa##aaa##."};
#endif

#ifdef USING_BinIcon_xpm32
static const char *const BinIcon_xpm32[]={
"32 32 3 1",
"# c #800000",
"a c #ff0000",
". c None",
"................................",
"...............##...............",
"...............##...............",
"...............##...............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
".............aa##aa.............",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"...........##aa##aa##...........",
"..........a##aa##aa##a..........",
"..........a##aa##aa##a..........",
"..........a##aa##aa##a..........",
"..........a##aa##aa##a..........",
"..........a##aa##aa##a..........",
"..........a##aa##aa##a..........",
"........##a##aa##aa##a##........",
"........##a##aa##aa##a##........",
"........##a##aa##aa##a##........",
"......aa##a##aa##aa##a##aa......",
"##aa##aa##a##aa##aa##a##aa##aa#.",
"##aa##aa##a##aa##aa##a##aa##aa#."};
#endif

#ifdef USING_BinIcon_xpm48
static const char *const BinIcon_xpm48[]={
"48 48 4 1",
"a c #0000ff",
"# c #800000",
"b c #ff0000",
". c None",
"................................................",
".......................##.......................",
".......................aa.......................",
".......................a#a......................",
"......................a##a......................",
"......................a##.a.....................",
"....................bab##bab....................",
"....................bab##bab....................",
"....................bab##bab....................",
"....................abb##bba....................",
"....................abb##bba....................",
"....................abb##bba....................",
"....................abb##bba....................",
"...................abbb##bbba...................",
"...................abbb##bbba...................",
"...................abbb##bbba...................",
"...................abbb##bbba...................",
"..................a.bbb##bbb.a..................",
".................#a#bbb##bbb#a#.................",
".................#a#bbb##bbb#a#.................",
".................#a#bbb##bbb#a#.................",
".................#a#bbb##bbb#a#.................",
".................a##bbb##bbb##a.................",
".................a##bbb##bbb##a.................",
".................a##bbb##bbb##a.................",
".................a##bbb##bbb##a.................",
".................a##bbb##bbb##a.................",
"................a###bbb##bbb###a................",
"................a###bbb##bbb###a................",
"................a###bbb##bbb###a................",
"................a###bbb##bbb###a................",
"..............bab###bbb##bbb###abb..............",
"..............bab###bbb##bbb###bab..............",
"..............bab###bbb##bbb###bab..............",
"..............bab###bbb##bbb###bab..............",
"..............abb###bbb##bbb###bab..............",
"..............abb###bbb##bbb###bba..............",
"..............abb###bbb##bbb###bba..............",
"..............abb###bbb##bbb###bba..............",
".............abbb###bbb##bbb###bbba.............",
"............#abbb###bbb##bbb###bbba#............",
"............#abbb###bbb##bbb###bbb#a............",
"............a#bbb###bbb##bbb###bbb#a............",
"............a#bbb###bbb##bbb###bbb##a...........",
"...........a##bbb###bbb##bbb###bbb##a...........",
".........bab##bbb###bbb##bbb###bbb##bab.........",
"###bbb###abb##bbb###bbb##bbb###bbb##bba###bbb##.",
"aaaaaaaaabbb##bbb###bbb##bbb###bbb##bbbaaaaaaaaa"};
#endif

#ifdef USING_BinIcon_xpm64
static const char *const BinIcon_xpm64[]={
"64 64 4 1",
"a c #0000ff",
"# c #800000",
"b c #ff0000",
". c None",
"................................................................",
"..............................####..............................",
"..............................#aa#..............................",
"..............................a##a..............................",
"..............................a##a..............................",
".............................a####a.............................",
".............................a####a.............................",
"............................a.####.a............................",
"..........................bbab####babb..........................",
"..........................bbab####babb..........................",
"..........................bbab####babb..........................",
"..........................babb####bbab..........................",
"..........................babb####bbab..........................",
"..........................babb####bbab..........................",
"..........................babb####bbab..........................",
"..........................abbb####bbba..........................",
"..........................abbb####bbba..........................",
"..........................abbb####bbba..........................",
"..........................abbb####bbba..........................",
".........................abbbb####bbbba.........................",
".........................abbbb####bbbba.........................",
".........................abbbb####bbbba.........................",
".........................abbbb####bbbba.........................",
".........................abbbb####bbbba.........................",
"........................a.bbbb####bbbb.a........................",
".......................#a#bbbb####bbbb#a#.......................",
".......................#a#bbbb####bbbb#a#.......................",
".......................#a#bbbb####bbbb#a#.......................",
".......................#a#bbbb####bbbb#a#.......................",
".......................a##bbbb####bbbb##a.......................",
".......................a##bbbb####bbbb##a.......................",
".......................a##bbbb####bbbb##a.......................",
".......................a##bbbb####bbbb##a.......................",
".......................a##bbbb####bbbb##a.......................",
"......................a###bbbb####bbbb###a......................",
"......................a###bbbb####bbbb###a......................",
"......................a###bbbb####bbbb###a......................",
"......................a###bbbb####bbbb###a......................",
"......................a###bbbb####bbbb###a......................",
".....................a.###bbbb####bbbb###.a.....................",
".....................a.###bbbb####bbbb###.a.....................",
".....................a.###bbbb####bbbb###.a.....................",
"...................bbab###bbbb####bbbb###babb...................",
"...................bbab###bbbb####bbbb###bbab...................",
"...................babb###bbbb####bbbb###bbab...................",
"...................babb###bbbb####bbbb###bbab...................",
"...................babb###bbbb####bbbb###bbab...................",
"...................babb###bbbb####bbbb###bbba...................",
"...................abbb###bbbb####bbbb###bbba...................",
"...................abbb###bbbb####bbbb###bbba...................",
"...................abbb###bbbb####bbbb###bbba...................",
"..................abbbb###bbbb####bbbb###bbbba..................",
"..................abbbb###bbbb####bbbb###bbbba..................",
".................a.bbbb###bbbb####bbbb###bbbba..................",
"...............##a#bbbb###bbbb####bbbb###bbbb#a##...............",
"...............##a#bbbb###bbbb####bbbb###bbbb#a##...............",
"...............#a##bbbb###bbbb####bbbb###bbbb##a#...............",
"...............#a##bbbb###bbbb####bbbb###bbbb##a#...............",
"...............a###bbbb###bbbb####bbbb###bbbb###a...............",
"..............a####bbbb###bbbb####bbbb###bbbb###a...............",
"............bba####bbbb###bbbb####bbbb###bbbb####abb............",
"............bab####bbbb###bbbb####bbbb###bbbb####bab............",
"####bbbb###aabb####bbbb###bbbb####bbbb###bbbb####bbaa###bbbb###.",
"aaaaaaaaaaa#bbb####bbbb###bbbb####bbbb###bbbb####bbb#aaaaaaaaaaa"};
#endif

#endif //BINOMIAL_H_INCLUDED
